// Script assets have changed for v2.3.0 see
// https://help.yoyogames.com/hc/en-us/articles/360005277377 for more information
function scr_3d_model(){

}


/*
//===================== Function List ==========================

//------------- Use System

draw_3d_start()
draw_3d_end()

//------------- Draw Type

draw_3d_vertex_normal(vertex_buffer,x,y,z,xn,yn,zn,xtex,ytex)

draw_3d_floor(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat)
draw_3d_wall(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat)
draw_3d_block(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat)
draw_3d_block_ext(x1,y1,z1,x2,y2,z2,tex,xrepeat,yrepeat,zrepeat)
draw_3d_cone(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat,closed,steps)
draw_3d_cylinder(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat,closed,steps)
draw_3d_ellipsoid(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat,steps)

//------------- Model Type

model_3d_build_begin(vbuff)
model_3d_build_end(vbuff)
model_3d_vertex_buffer_create()
model_3d_vertex_buffer_destroy(vbuff)

model_3d_draw(vbuff,x,y,z,tex)

model_3d_floor(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat)
model_3d_wall(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat)
model_3d_block(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat)
model_3d_block_ext(vbuff,x1,y1,z1,x2,y2,z2,xrepeat,yrepeat,zrepeat)
model_3d_cone(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat,closed,steps)
model_3d_cylinder(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat,closed,steps)
model_3d_ellipsoid(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat,steps)


//------------- Example ---------------
//----- Create Event
model_01=model_3d_vertex_buffer_create();

model_3d_build_begin(model_01);
model_3d_block(model_01,-16,-16,0,16,16,32,1,1);
model_3d_build_end(model_01);

//----- Draw Event
var tex=sprite_get_texture(spr_tex,0);
model_3d_draw(model_01,x,y,z,tex);

//----- Destroy Event or Game End
model_3d_vertex_buffer_destroy(model_01);

//=================== Function List END =======================
*/


///draw_3d_start();
function draw_3d_start() {

	vertex_format_begin();
	vertex_format_add_position_3d();//x,y,z
	vertex_format_add_normal();//nx,ny,nz
	vertex_format_add_texcoord();//xtex,ytex
	vertex_format_add_colour();//color,alpha
	global.__x_mod_vertex_format=vertex_format_end();
	global.__x_mod_vertex_buffer=vertex_create_buffer();

}

///draw_3d_end()
function draw_3d_end() {

	vertex_format_delete(global.__x_mod_vertex_format);
	vertex_delete_buffer(global.__x_mod_vertex_buffer);

}

///draw_3d_vertex_normal(vertex_buffer,x,y,z,xn,yn,zn,xtex,ytex)
///@param vertext_buffer
///@param x
///@param y
///@param z
///@param xn
///@param yn
///@param zn
///@param xtex
///@param ytex
function draw_3d_vertex_normal(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {

	var vbuff,_x,_y,_z,nx,ny,nz,u,v,col,alp;
	vbuff=argument0;
	_x=argument1;
	_y=argument2;
	_z=argument3;
	nx=argument4;
	ny=argument5;
	nz=argument6;
	u=argument7;
	v=argument8;
	col=draw_get_color();
	alp=draw_get_alpha();

	vertex_position_3d(vbuff,_x,_y,_z);
	vertex_normal(vbuff,nx,ny,nz);
	vertex_texcoord(vbuff,u,v);
	vertex_color(vbuff,col,alp);



}


//============================================================
//====================== Draw Type ===========================
//============================================================


/// draw_3d_floor(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat)
/// @description Draws a simple 3D floor.
/// @param x1 The initial x coordinate of the floor.
/// @param y1 The initial y coordinate of the floor.
/// @param z1 The initial z coordinate of the floor.
/// @param x2 The opposite x coordinate of the floor.
/// @param y2 The opposite y coordinate of the floor. 
/// @param z2 The opposite z coordinate of the floor.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @returns 
function draw_3d_floor(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {

	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;

	var __xdiff = __x2 - __x1;
	var __zdiff = __z2 - __z1;

	var __lsquared = (__xdiff * __xdiff) + (__zdiff * __zdiff);
	if (__lsquared == 0)
		return 0;
	
	var __l = sqrt(__lsquared);

	var __nx = -__zdiff / __l;
	var __nz = __xdiff / __l;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)

	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	vertex_begin(vbuff,global.__x_mod_vertex_format);
	//=====================

	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, __nx, 0, __nz, __uvx, __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, __nx, 0, __nz, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, __nx, 0, __nz, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, __nx, 0, __nz, __uvx+(__uvw*__hrepeat), __uvy);

	//=====================
	vertex_end(vbuff);
	vertex_submit(vbuff,pr_trianglefan,__tex);
	//=====================	

	gpu_set_texrepeat(__oldrep);


}

/// draw_3d_wall(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat)
/// @description Draws a simple 3D vertical wall.
/// @param x1 The initial x coordinate of the wall.
/// @param y1 The initial y coordinate of the wall.
/// @param z1 The initial z coordinate of the wall.
/// @param x2 The opposite x coordinate of the wall.
/// @param y2 The opposite y coordinate of the wall. 
/// @param z2 The opposite z coordinate of the wall.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @returns 
function draw_3d_wall(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {


	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;

	var __xdiff = __x2 - __x1;
	var __ydiff = __y2 - __y1;

	var __lsquared = (__xdiff * __xdiff) + (__ydiff * __ydiff);
	if (__lsquared == 0)
		return 0;
	
	var __l = sqrt(__lsquared);

	var __nx = __ydiff / __l;
	var __ny = -__xdiff / __l;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)	


	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	vertex_begin(vbuff,global.__x_mod_vertex_format);
	//=====================

	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, __nx, __ny, 0, __uvx, __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, __nx, __ny, 0, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, __nx, __ny, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, __nx, __ny, 0, __uvx, __uvy+(__uvh*__vrepeat));

	//=====================
	vertex_end(vbuff);
	vertex_submit(vbuff,pr_trianglefan,__tex);
	//=====================	
	
	gpu_set_texrepeat(__oldrep);

}

/// draw_3d_block(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat)
/// @description Draws a simple 3D block.
/// @param x1 The initial x coordinate of the block. 
/// @param y1 The initial y coordinate of the block.
/// @param z1 The initial z coordinate of the block.
/// @param x2 The opposite x coordinate of the block.
/// @param y2 The opposite y coordinate of the block.
/// @param z2 The opposite z coordinate of the block.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @returns 
function draw_3d_block(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {

	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)	

	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	vertex_begin(vbuff,global.__x_mod_vertex_format);
	//=====================


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 0, -1, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, 0, -1, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, 0, 1, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 0, 1, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 1, 0, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 1, 0, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 1, 0, 0, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 1, 0, 0, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, -1, 0, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));

	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, -1, 0, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, -1, 0, 0, __uvx, __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));

	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, -1, 0, 0, __uvx+(__uvw*__hrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, __uvx,__uvy);


	//=====================
	vertex_end(vbuff);
	vertex_submit(vbuff,pr_trianglelist,__tex);
	//=====================

	gpu_set_texrepeat(__oldrep);

}

/// draw_3d_block_ext(x1,y1,z1,x2,y2,z2,tex,xrepeat,yrepeat,zrepeat)
/// @description Draws a simple 3D block.
/// @param x1 The initial x coordinate of the block. 
/// @param y1 The initial y coordinate of the block.
/// @param z1 The initial z coordinate of the block.
/// @param x2 The opposite x coordinate of the block.
/// @param y2 The opposite y coordinate of the block.
/// @param z2 The opposite z coordinate of the block.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param xrepeat Amount of x repetitions for the texture.
/// @param yrepeat Amount of y repetitions for the texture.
/// @param zrepeat Amount of z repetitions for the texture.
/// @returns 
function draw_3d_block_ext(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9) {

	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __xrepeat = argument7;
	var __yrepeat = argument8;
	var __zrepeat = argument9;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)	

	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	vertex_begin(vbuff,global.__x_mod_vertex_format);
	//=====================


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 0, -1, __uvx, __uvy+(__uvh*__yrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__yrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__yrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, 0, -1, __uvx+(__uvw*__xrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, 0, 1, __uvx+(__uvw*__xrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__yrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__yrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 0, 1, __uvx, __uvy+(__uvh*__yrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 1, 0, __uvx, __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__zrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 1, 0, __uvx+(__uvw*__xrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 1, 0, 0, __uvx, __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __uvx+(__uvw*__yrepeat), __uvy+(__uvh*__zrepeat));

	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __uvx+(__uvw*__yrepeat), __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 1, 0, 0, __uvx+(__uvw*__yrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, -1, 0, __uvx, __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__zrepeat));

	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __uvx+(__uvw*__xrepeat), __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, -1, 0, __uvx+(__uvw*__xrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, __uvx,__uvy);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, __uvx,__uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, -1, 0, 0, __uvx, __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __uvx+(__uvw*__yrepeat), __uvy+(__uvh*__zrepeat));

	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __uvx+(__uvw*__yrepeat), __uvy+(__uvh*__zrepeat));
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, -1, 0, 0, __uvx+(__uvw*__yrepeat), __uvy);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, __uvx,__uvy);


	//=====================
	vertex_end(vbuff);
	vertex_submit(vbuff,pr_trianglelist,__tex);
	//=====================

	gpu_set_texrepeat(__oldrep);

}

/// draw_3d_cone(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat,closed,steps)
/// @description Draws a simple 3D cone.
/// @param x1 The initial x coordinate of the cone.
/// @param y1 The initial y coordinate of the cone.
/// @param z1 The initial z coordinate of the cone.
/// @param x2 The opposite x coordinate of the cone.
/// @param y2 The opposite y coordinate of the cone. 
/// @param z2 The opposite z coordinate of the cone.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @param closed Sets whether to close (true) the bottom of the cone or not (false).
/// @param steps How many steps are used to make the cone "round" (typically 24)
/// @returns 
function draw_3d_cone(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9, argument10) {

	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;
	var __closed = argument9;
	var __steps = argument10;

	if (__steps < 3)
	{
		__steps = 3;
	}

	if (__steps > 128)
	{
		__steps = 128;
	}

	// Create sin and cos tables
	var __cc;
	var __ss;

	__cc[__steps] = 0;
	__ss[__steps] = 0;

	var __i;
	for(__i = 0; __i <= __steps; __i++)
	{
		var __rad = (__i * 2.0 * pi) / __steps;
		__cc[__i] = cos(__rad);
		__ss[__i] = sin(__rad);
	}

	var __mx = (__x2 + __x1) / 2;
	var __my = (__y2 + __y1) / 2;
	var __rx = (__x2 - __x1) / 2;
	var __ry = (__y2 - __y1) / 2;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)	

	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	vertex_begin(vbuff,global.__x_mod_vertex_format);
	//=====================

	for(__i = 0; __i <= __steps; __i++)
	{
		draw_3d_vertex_normal(vbuff,__mx, __my, __z2, 0, 0, 1, __uvx+(__uvw*(__hrepeat * __i / __steps)), __uvy+(__uvh*__vrepeat));
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z1, __cc[__i] , __ss[__i] ,0 , __uvx+(__uvw*(__hrepeat * __i / __steps)), __uvy);
	}


	//=====================
	vertex_end(vbuff);
	vertex_submit(vbuff,pr_trianglestrip,__tex);
	//=====================

	if (__closed == true)
	{
		//=====================
		vertex_begin(vbuff,global.__x_mod_vertex_format);
		//=====================
	
	
		draw_3d_vertex_normal(vbuff,__mx, __my, __z1, 0, 0, -1, __uvx, __uvy);
		for(__i = __steps; __i >= 0; __i--)
		{
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z1, 0, 0, -1, __uvx, __uvy);		
		}
		//=====================
		vertex_end(vbuff);
		vertex_submit(vbuff,pr_trianglefan,__tex);
		//=====================
	}

	gpu_set_texrepeat(__oldrep);

}

/// draw_3d_cylinder(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat,closed,steps)
/// @description Draws a simple 3D cylinder.
/// @param x1 The initial x coordinate of the cylinder.
/// @param y1 The initial y coordinate of the cylinder.
/// @param z1 The initial z coordinate of the cylinder.
/// @param x2 The opposite x coordinate of the cylinder.
/// @param y2 The opposite y coordinate of the cylinder. 
/// @param z2 The opposite z coordinate of the cylinder.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @param closed Sets whether to close (true) the bottom and top of the cylinder or not (false).
/// @param steps How many steps are used to make the cylinder "round" (typically 24)
/// @returns 
function draw_3d_cylinder(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9, argument10) {

	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;
	var __closed = argument9;
	var __steps = argument10;

	if (__steps < 3)
	{
		__steps = 3;
	}

	if (__steps > 128)
	{
		__steps = 128;
	}

	// Create sin and cos tables
	var __cc;
	var __ss;

	__cc[__steps] = 0;
	__ss[__steps] = 0;

	var __i;
	for(__i = 0; __i <= __steps; __i++)
	{
		var __rad = (__i * 2.0 * pi) / __steps;
		__cc[__i] = cos(__rad);
		__ss[__i] = sin(__rad);
	}

	var __mx = (__x2 + __x1) / 2;
	var __my = (__y2 + __y1) / 2;
	var __rx = (__x2 - __x1) / 2;
	var __ry = (__y2 - __y1) / 2;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)	

	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	//=====================


	if (__closed == true)
	{
		//=====================
		vertex_begin(vbuff,global.__x_mod_vertex_format);
		//=====================
		draw_3d_vertex_normal(vbuff,__mx, __my, __z2, 0, 0, 1, __uvx, __uvy+(__uvh*__vrepeat));
		for(__i = 0; __i <= __steps; __i++)
		{
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z2, 0, 0, 1, __uvx, __uvy+(__uvh*__vrepeat));		
		}
	
		//=====================
		vertex_end(vbuff);vertex_submit(vbuff,pr_trianglefan,__tex);
		//=====================	
	}


	//=====================
	vertex_begin(vbuff,global.__x_mod_vertex_format);
	//=====================

	for(__i = 0; __i <= __steps; __i++)
	{
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z2, __cc[__i] , __ss[__i] ,0 , __uvx+(__uvw*(__hrepeat * __i / __steps)), __uvy+(__uvh*__vrepeat));
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z1, __cc[__i] , __ss[__i] ,0 , __uvx+(__uvw*(__hrepeat * __i / __steps)), __uvy);
	}

	//=====================
	vertex_end(vbuff);vertex_submit(vbuff,pr_trianglestrip,__tex);
	//=====================	
	

	if (__closed == true)
	{
		//=====================
		vertex_begin(vbuff,global.__x_mod_vertex_format);
		//=====================
		draw_3d_vertex_normal(vbuff,__mx, __my, __z1, 0, 0, -1, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));
		for(__i = __steps; __i >= 0; __i--)
		{
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z1, 0, 0, -1, __uvx+(__uvw*__hrepeat), __uvy+(__uvh*__vrepeat));		
		}
		//=====================
		vertex_end(vbuff);vertex_submit(vbuff,pr_trianglefan,__tex);
		//=====================	
	}

	gpu_set_texrepeat(__oldrep);

}

/// draw_3d_ellipsoid(x1,y1,z1,x2,y2,z2,tex,hrepeat,vrepeat,steps)
/// @description Draws a simple 3D ellipsoid.
/// @param x1 The initial x coordinate of the ellipsoid.
/// @param y1 The initial y coordinate of the ellipsoid.
/// @param z1 The initial z coordinate of the ellipsoid.
/// @param x2 The opposite x coordinate of the ellipsoid.
/// @param y2 The opposite y coordinate of the ellipsoid. 
/// @param z2 The opposite z coordinate of the ellipsoid.
/// @param tex The id of the texture to use (-1 for no texture)
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @param steps How many steps are used to make the ellipsoid "round" (typically 24)
/// @returns 
function draw_3d_ellipsoid(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9) {

	var __x1 = argument0;
	var __y1 = argument1;
	var __z1 = argument2;
	var __x2 = argument3;
	var __y2 = argument4;
	var __z2 = argument5;
	var __tex = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;
	var __steps = argument9;

	if (__steps < 3)
	{
		__steps = 3;
	}

	if (__steps > 128)
	{
		__steps = 128;
	}

	// Create sin and cos tables
	var __cc;
	var __ss;

	__cc[__steps] = 0;
	__ss[__steps] = 0;

	var __i;
	for(__i = 0; __i <= __steps; __i++)
	{
		var __rad = (__i * 2.0 * pi) / __steps;
		__cc[__i] = cos(__rad);
		__ss[__i] = sin(__rad);
	}

	var __mx = (__x2 + __x1) / 2;
	var __my = (__y2 + __y1) / 2;
	var __mz = (__z2 + __z1) / 2;
	var __rx = (__x2 - __x1) / 2;
	var __ry = (__y2 - __y1) / 2;
	var __rz = (__z2 - __z1) / 2;

	var __rows = (__steps + 1) / 2;
	var __j;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true)	


	//=====================
	var __uvx,__uvy,__uvw,__uvh,__uvs;
	__uvs=texture_get_uvs(__tex);
	__uvx=__uvs[0];
	__uvy=__uvs[1];
	__uvw=__uvs[2] - __uvs[0];
	__uvh=__uvs[3] - __uvs[1];
	var vbuff=global.__x_mod_vertex_buffer;
	//=====================

	for(__j = 0; __j <= (__rows - 1); __j++)
	{
		var __row1rad = (__j*pi)/__rows;
		var __row2rad = ((__j+1)*pi)/__rows;
		var __rh1 = cos(__row1rad);
		var __rd1 = sin(__row1rad);
		var __rh2 = cos(__row2rad);
		var __rd2 = sin(__row2rad);
	
		vertex_begin(vbuff,global.__x_mod_vertex_format);
	
		for(__i = 0; __i <= __steps; __i++)
		{
			// Some common subexpressions here that could be eliminated
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd1*__cc[__i], __my+__ry*__rd1*__ss[__i], __mz+__rz*__rh1,__rd1*__cc[__i], __rd1*__ss[__i], __rh1, __uvx+(__uvw*(__hrepeat*__i/__steps)), __uvy+(__uvh*(__j*__vrepeat/__rows)));
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd2*__cc[__i], __my+__ry*__rd2*__ss[__i], __mz+__rz*__rh2,__rd2*__cc[__i], __rd2*__ss[__i], __rh2, __uvx+(__uvw*(__hrepeat*__i/__steps)), __uvy+(__uvh*((__j+1)*__vrepeat/__rows)));
		}
	
		//=====================
		vertex_end(vbuff);
		vertex_submit(vbuff,pr_trianglestrip,__tex);
		//=====================	
	}

	gpu_set_texrepeat(__oldrep);

}

//============================================================
//====================== Draw Type END ======================
//============================================================


//============================================================
//====================== Model Type ==========================
//============================================================

///model_3d_build_begin(vbuff)
/// @param vbuff vertex buffer. 
function model_3d_build_begin(argument0) {
	var vbuff=argument0;
	vertex_begin(vbuff,global.__x_mod_vertex_format);
}

///model_3d_build_end(vbuff)
/// @param vbuff vertex buffer. 
function model_3d_build_end(argument0) {
	var vbuff=argument0;
	vertex_end(vbuff);
}

///model_3d_vertex_buffer_create()
function model_3d_vertex_buffer_create() {
	return vertex_create_buffer();
}

///model_3d_vertex_buffer_destroy(vbuff)
/// @param vbuff vertex buffer.
function model_3d_vertex_buffer_destroy(argument0) {
	vertex_delete_buffer(argument0);
}

///model_3d_draw(vbuff,x,y,z,tex)
/// @param vbuff vertex buffer.
/// @param x
/// @param y
/// @param z
/// @param tex texture
function model_3d_draw(argument0, argument1, argument2, argument3, argument4) {
	var vbuff,_x,_y,_z,tex;
	vbuff=argument0;
	_x=argument1;
	_y=argument2;
	_z=argument3;
	tex=argument4;

	var __oldrep = gpu_get_texrepeat();
	gpu_set_texrepeat(true);

	matrix_set(matrix_world,matrix_multiply(matrix_get(matrix_world),matrix_build(_x,_y,_z,0,0,0,1,1,1)));
	if vertex_get_number(vbuff)>0{vertex_submit(vbuff,pr_trianglelist,tex);}
	matrix_set(matrix_world,matrix_build_identity());

	gpu_set_texrepeat(__oldrep);

}

/// model_3d_floor(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat)
/// @description Draws a simple 3D floor.
/// @param vbuff vertex buffer. 
/// @param x1 The initial x coordinate of the floor.
/// @param y1 The initial y coordinate of the floor.
/// @param z1 The initial z coordinate of the floor.
/// @param x2 The opposite x coordinate of the floor.
/// @param y2 The opposite y coordinate of the floor. 
/// @param z2 The opposite z coordinate of the floor.
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @returns 
function model_3d_floor(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {
	var vbuff=argument0;

	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;

	var __xdiff = __x2 - __x1;
	var __zdiff = __z2 - __z1;

	var __lsquared = (__xdiff * __xdiff) + (__zdiff * __zdiff);
	if (__lsquared == 0)
		return 0;
	
	var __l = sqrt(__lsquared);

	var __nx = -__zdiff / __l;
	var __nz = __xdiff / __l;


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, __nx, 0, __nz, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, __nx, 0, __nz, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, __nx, 0, __nz, __hrepeat, 0);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, __nx, 0, __nz, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, __nx, 0, __nz, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, __nx, 0, __nz, 0, __vrepeat);

}

/// model_3d_wall(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat)
/// @description Draws a simple 3D vertical wall.
/// @param vbuff vertex buffer. 
/// @param x1 The initial x coordinate of the wall.
/// @param y1 The initial y coordinate of the wall.
/// @param z1 The initial z coordinate of the wall.
/// @param x2 The opposite x coordinate of the wall.
/// @param y2 The opposite y coordinate of the wall. 
/// @param z2 The opposite z coordinate of the wall.
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @returns 
function model_3d_wall(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {

	var vbuff=argument0;

	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;

	var __xdiff = __x2 - __x1;
	var __ydiff = __y2 - __y1;

	var __lsquared = (__xdiff * __xdiff) + (__ydiff * __ydiff);
	if (__lsquared == 0)
		return 0;
	
	var __l = sqrt(__lsquared);

	var __nx = __ydiff / __l;
	var __ny = -__xdiff / __l;


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, __nx, __ny, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, __nx, __ny, 0, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, __nx, __ny, 0, 0, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, __nx, __ny, 0, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, __nx, __ny, 0, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, __nx, __ny, 0, __hrepeat, 0);

}

/// model_3d_block(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat)
/// @description Draws a simple 3D block.
/// @param vbuff vertex buffer. 
/// @param x1 The initial x coordinate of the block. 
/// @param y1 The initial y coordinate of the block.
/// @param z1 The initial z coordinate of the block.
/// @param x2 The opposite x coordinate of the block.
/// @param y2 The opposite y coordinate of the block.
/// @param z2 The opposite z coordinate of the block.
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @returns 
function model_3d_block(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8) {

	var vbuff=argument0;

	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;



	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 0, -1, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __hrepeat, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, 0, -1, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, 0, 0);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, 0, 1, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __hrepeat, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 0, 1, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, 0, 0);


	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 1, 0, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __hrepeat, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 1, 0, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, 0, 0);


	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 1, 0, 0, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __hrepeat, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 1, 0, 0, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, 0, 0);


	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, -1, 0, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __hrepeat, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, -1, 0, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, 0, 0);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, -1, 0, 0, 0, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __hrepeat, __vrepeat);

	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __hrepeat, __vrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, -1, 0, 0, __hrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, 0, 0);

}

/// model_3d_block_ext(vbuff,x1,y1,z1,x2,y2,z2,xrepeat,yrepeat,zrepeat)
/// @description Draws a simple 3D block.
/// @param vbuff vertex buffer. 
/// @param x1 The initial x coordinate of the block. 
/// @param y1 The initial y coordinate of the block.
/// @param z1 The initial z coordinate of the block.
/// @param x2 The opposite x coordinate of the block.
/// @param y2 The opposite y coordinate of the block.
/// @param z2 The opposite z coordinate of the block.
/// @param xrepeat Amount of x repetitions for the texture.
/// @param yrepeat Amount of y repetitions for the texture.
/// @param zrepeat Amount of z repetitions for the texture.
/// @returns 
function model_3d_block_ext(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9) {

	var vbuff=argument0;

	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __xrepeat = argument7;
	var __yrepeat = argument8;
	var __zrepeat = argument9;



	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 0, -1, 0, __yrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __xrepeat, __yrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 0, -1, __xrepeat, __yrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, 0, -1, __xrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, 0, -1, 0, 0);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, 0, 1, __xrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __xrepeat, __yrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 0, 1, __xrepeat, __yrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 0, 1, 0, __yrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, 0, 1, 0, 0);


	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, 0, 1, 0, 0, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __xrepeat, __zrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 0, 1, 0, __xrepeat, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 0, 1, 0, __xrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, 0, 1, 0, 0, 0);


	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z2, 1, 0, 0, 0, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __yrepeat, __zrepeat);

	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 1, 0, 0, __yrepeat, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 1, 0, 0, __yrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y2, __z1, 1, 0, 0, 0, 0);


	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z2, 0, -1, 0, 0, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __xrepeat, __zrepeat);

	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, 0, -1, 0, __xrepeat, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, 0, -1, 0, __xrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x2, __y1, __z1, 0, -1, 0, 0, 0);


	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, 0, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z2, -1, 0, 0, 0, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __yrepeat, __zrepeat);

	draw_3d_vertex_normal(vbuff,__x1, __y2, __z2, -1, 0, 0, __yrepeat, __zrepeat);
	draw_3d_vertex_normal(vbuff,__x1, __y2, __z1, -1, 0, 0, __yrepeat, 0);
	draw_3d_vertex_normal(vbuff,__x1, __y1, __z1, -1, 0, 0, 0, 0);

}

/// model_3d_cone(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat,closed,steps)
/// @description Draws a simple 3D cone.
/// @param vbuff vertex buffer. 
/// @param x1 The initial x coordinate of the cone.
/// @param y1 The initial y coordinate of the cone.
/// @param z1 The initial z coordinate of the cone.
/// @param x2 The opposite x coordinate of the cone.
/// @param y2 The opposite y coordinate of the cone. 
/// @param z2 The opposite z coordinate of the cone.
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @param closed Sets whether to close (true) the bottom of the cone or not (false).
/// @param steps How many steps are used to make the cone "round" (typically 24)
/// @returns 
function model_3d_cone(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9, argument10) {

	var vbuff=argument0;
	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;
	var __closed = argument9;
	var __steps = argument10;

	if (__steps < 3)
	{
		__steps = 3;
	}

	if (__steps > 128)
	{
		__steps = 128;
	}

	// Create sin and cos tables
	var __cc;
	var __ss;

	__cc[__steps] = 0;
	__ss[__steps] = 0;

	var __i;
	for(__i = 0; __i <= __steps; __i++)
	{
		var __rad = (__i * 2.0 * pi) / __steps;
		__cc[__i] = cos(__rad);
		__ss[__i] = sin(__rad);
	}

	var __mx = (__x2 + __x1) / 2;
	var __my = (__y2 + __y1) / 2;
	var __rx = (__x2 - __x1) / 2;
	var __ry = (__y2 - __y1) / 2;
	var __on;



	__on=0;
	for(__i = 0; __i <= __steps; __i++)
	{
		if __on=1{
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z1, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, 0);
		}
	
	
		if __i<__steps{
		draw_3d_vertex_normal(vbuff,__mx, __my, __z2, 0, 0, 1, __hrepeat * __i / __steps, __vrepeat);
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z1, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, 0);
		}
	
		__on=1;
	}


	if (__closed == true)
	{
		__on=0;
		for(__i = __steps; __i >= 0; __i--)
		{
			if __on=1{
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z1, 0, 0, -1, 0, 0);
			}
		
			if __i>0{
			draw_3d_vertex_normal(vbuff,__mx, __my, __z1, 0, 0, -1, 0, 0);
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z1, 0, 0, -1, 0, 0);		
			}
			__on=1;
		}
	
	}


}

/// model_3d_cylinder(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat,closed,steps)
/// @description Draws a simple 3D cylinder.
/// @param x1 The initial x coordinate of the cylinder.
/// @param y1 The initial y coordinate of the cylinder.
/// @param z1 The initial z coordinate of the cylinder.
/// @param x2 The opposite x coordinate of the cylinder.
/// @param y2 The opposite y coordinate of the cylinder. 
/// @param z2 The opposite z coordinate of the cylinder.
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @param closed Sets whether to close (true) the bottom and top of the cylinder or not (false).
/// @param steps How many steps are used to make the cylinder "round" (typically 24)
/// @returns 
function model_3d_cylinder(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9, argument10) {

	var vbuff=argument0;

	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;
	var __closed = argument9;
	var __steps = argument10;

	if (__steps < 3)
	{
		__steps = 3;
	}

	if (__steps > 128)
	{
		__steps = 128;
	}

	// Create sin and cos tables
	var __cc;
	var __ss;

	__cc[__steps] = 0;
	__ss[__steps] = 0;

	var __i,__on;
	var _px,_py,_pz,_nx,_ny,_nz,_tx,_ty;



	for(__i = 0; __i <= __steps; __i++)
	{
		var __rad = (__i * 2.0 * pi) / __steps;
		__cc[__i] = cos(__rad);
		__ss[__i] = sin(__rad);
	}

	var __mx = (__x2 + __x1) / 2;
	var __my = (__y2 + __y1) / 2;
	var __rx = (__x2 - __x1) / 2;
	var __ry = (__y2 - __y1) / 2;


	if (__closed == true)
	{

		__on=0;
		for(__i = 0; __i <= __steps; __i++)
		{
			if __on=1{
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z2, 0, 0, 1, 0, __vrepeat);
			}
		
			if __i<__steps{
			draw_3d_vertex_normal(vbuff,__mx, __my, __z2, 0, 0, 1, 0, __vrepeat);
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z2, 0, 0, 1, 0, __vrepeat);		
			}
			__on=1;
		}
	
	}


	__on=0;

	for(__i = 0; __i <= __steps; __i++)
	{
		if __on=1{
	
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z2, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, __vrepeat);
	
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z1, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, 0);
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z2, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, __vrepeat);
		draw_3d_vertex_normal(vbuff,_px,_py,_pz,_nx,_ny,_nz,_tx,_ty);
		}
	
		if __i<__steps{
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z2, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, __vrepeat);
		draw_3d_vertex_normal(vbuff,__mx + __cc[__i]*__rx, __my + __ss[__i]*__ry, __z1, __cc[__i] , __ss[__i] ,0 , __hrepeat * __i / __steps, 0);
	
		_px=__mx + __cc[__i]*__rx;
		_py=__my + __ss[__i]*__ry;
		_pz=__z1;
		_nx=__cc[__i];
		_ny=__ss[__i];
		_nz=0;
		_tx=__hrepeat * __i / __steps;
		_ty= 0;
	
		}
		__on=1;
	
	}


	if (__closed == true)
	{
		__on=0;
		for(__i = __steps; __i >= 0; __i--)
		{
			if __on=1{
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z1, 0, 0, -1, 0, 0);
			}
		
			if __i>0{
			draw_3d_vertex_normal(vbuff,__mx, __my, __z1, 0, 0, -1, 0, 0);
			draw_3d_vertex_normal(vbuff,__mx+__cc[__i]*__rx, __my+__ss[__i]*__ry, __z1, 0, 0, -1, 0, 0);		
			}
			__on=1;
		}

	}

}

/// model_3d_ellipsoid(vbuff,x1,y1,z1,x2,y2,z2,hrepeat,vrepeat,steps)
/// @description Draws a simple 3D ellipsoid.
/// @param vbuff vertex buffer. 
/// @param x1 The initial x coordinate of the ellipsoid.
/// @param y1 The initial y coordinate of the ellipsoid.
/// @param z1 The initial z coordinate of the ellipsoid.
/// @param x2 The opposite x coordinate of the ellipsoid.
/// @param y2 The opposite y coordinate of the ellipsoid. 
/// @param z2 The opposite z coordinate of the ellipsoid.
/// @param hrepeat Amount of horizontal repetitions for the texture.
/// @param vrepeat Amount of vertical repetitions for the texture.
/// @param steps How many steps are used to make the ellipsoid "round" (typically 24)
/// @returns 
function model_3d_ellipsoid(argument0, argument1, argument2, argument3, argument4, argument5, argument6, argument7, argument8, argument9) {

	var vbuff=argument0;
	var __x1 = argument1;
	var __y1 = argument2;
	var __z1 = argument3;
	var __x2 = argument4;
	var __y2 = argument5;
	var __z2 = argument6;
	var __hrepeat = argument7;
	var __vrepeat = argument8;
	var __steps = argument9;

	if (__steps < 3)
	{
		__steps = 3;
	}

	if (__steps > 128)
	{
		__steps = 128;
	}

	// Create sin and cos tables
	var __cc;
	var __ss;

	__cc[__steps] = 0;
	__ss[__steps] = 0;

	var __i;
	for(__i = 0; __i <= __steps; __i++)
	{
		var __rad = (__i * 2.0 * pi) / __steps;
		__cc[__i] = cos(__rad);
		__ss[__i] = sin(__rad);
	}

	var __mx = (__x2 + __x1) / 2;
	var __my = (__y2 + __y1) / 2;
	var __mz = (__z2 + __z1) / 2;
	var __rx = (__x2 - __x1) / 2;
	var __ry = (__y2 - __y1) / 2;
	var __rz = (__z2 - __z1) / 2;

	var __rows = (__steps + 1) / 2;
	var __j;
	var __on;
	var _px,_py,_pz,_nx,_ny,_nz,_tx,_ty;


	for(__j = 0; __j <= (__rows - 1); __j++)
	{
		var __row1rad = (__j*pi)/__rows;
		var __row2rad = ((__j+1)*pi)/__rows;
		var __rh1 = cos(__row1rad);
		var __rd1 = sin(__row1rad);
		var __rh2 = cos(__row2rad);
		var __rd2 = sin(__row2rad);
	
	
		__on=0;
		for(__i = 0; __i <= __steps; __i++)
		{
			// Some common subexpressions here that could be eliminated
			if __on=1{
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd1*__cc[__i], __my+__ry*__rd1*__ss[__i], __mz+__rz*__rh1,__rd1*__cc[__i], __rd1*__ss[__i], __rh1, __hrepeat*__i/__steps, __j*__vrepeat/__rows);
		
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd2*__cc[__i], __my+__ry*__rd2*__ss[__i], __mz+__rz*__rh2,__rd2*__cc[__i], __rd2*__ss[__i], __rh2, __hrepeat*__i/__steps, (__j+1)*__vrepeat/__rows);
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd1*__cc[__i], __my+__ry*__rd1*__ss[__i], __mz+__rz*__rh1,__rd1*__cc[__i], __rd1*__ss[__i], __rh1, __hrepeat*__i/__steps, __j*__vrepeat/__rows);
			draw_3d_vertex_normal(vbuff,_px,_py,_pz,_nx,_ny,_nz,_tx,_ty);
			}
		
			if __i<__steps{
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd1*__cc[__i], __my+__ry*__rd1*__ss[__i], __mz+__rz*__rh1,__rd1*__cc[__i], __rd1*__ss[__i], __rh1, __hrepeat*__i/__steps, __j*__vrepeat/__rows);
			draw_3d_vertex_normal(vbuff,__mx+__rx*__rd2*__cc[__i], __my+__ry*__rd2*__ss[__i], __mz+__rz*__rh2,__rd2*__cc[__i], __rd2*__ss[__i], __rh2, __hrepeat*__i/__steps, (__j+1)*__vrepeat/__rows);
		
			_px=__mx+__rx*__rd2*__cc[__i];
			_py=__my+__ry*__rd2*__ss[__i];
			_pz=__mz+__rz*__rh2;
			_nx=__rd2*__cc[__i];
			_ny=__rd2*__ss[__i];
			_nz=__rh2;
			_tx=__hrepeat*__i/__steps;
			_ty=(__j+1)*__vrepeat/__rows;
			}
			__on=1;
		}
	
	
	}

}

//============================================================
//====================== Model Type END =====================
//============================================================
